@@ -1,3 +1,16 @@
+#' Dummy sample function
+#'
+#' @param x A vector from which to sample
+#' @param ... All other arguments in sample
+#'
+#' @return The randomly sampled items
+#' @keywords internal
+resample <- function(x, ...){
+  x[sample.int(length(x), ...)]
+}
+
+
+
 #' Reduce haplos to contain non-cSNV data
 #'
 #' @inheritParams sim_FGenos
@@ -11,74 +24,100 @@ condition_haplos_no_cSNV <- function(haplos, RV_pool_loc){
     which(haplos[, x] == 1)
   })
 
+  #RV rows
   RV_rows <- Reduce(union, RV_haps)
 
   return(haplos[-RV_rows, ])
 }
 
+#' Find haplotypes that do not carry any cRVs
+#'
+#' @inheritParams sim_FGenos
+#'
+#' @return The reduced haplotype matrix.
+#'
+#' @keywords internal
+find_no_cSNV_rows <- function(haplos, RV_pool_loc){
+  #determine which of the haplotypes carry a causal SNV
+  RV_haps <- lapply(RV_pool_loc, function(x){
+    which(haplos[, x] == 1)
+  })
+
+  #RV rows
+  any_RV_rows <- Reduce(union, RV_haps)
+
+  #no RV rows
+  no_RV_rows <- setdiff(1:nrow(haplos), any_RV_rows)
+
+  return(no_RV_rows)
+}
+
 #' Draw Founder Genotypes from Haplotype Distribution Given Familial Risk Variant
 #'
 #' \strong{For internal use.}
 #'
-#' @param founder_ids Numeric list. The ID numbers of all non-seed founders.
-#' @param RV_founder Numeric. The ID number of the seed founder.
-#' @param RV_founder_pat Numeric. RV_founder_pat == 1 if RV founder inherited the RV from dad, and 0 if inherited RV from mom.
+#' @param founder_ids Numeric list. The ID numbers of all founders.
+#' @param RV_founder Numeric. The ID number(s) of the founder who introduced a cRV.
+#' @param founder_pat_allele  Numeric list. The paternally inherited allele for each founder.
+#' @param founder_mat_allele  Numeric list. The maternally inherited allele for each founder.
 #' @param haplos sparseMatrix.  The sparseMatrix of genomes returned by \code{read_slimOut}.
 #' @param RV_col_loc Numeric. The column location of the familial RV in haplos.
-#' @param RV_pool_loc The column locations of each SNV in the pool of candidate SNVs.
+#' @param RV_pool_loc The column locations of each SNV (in haplos)  for SNVs in the pool of cRVs.
 #'
 #' @return list of familial founder genotypes
 #'
 #' @keywords internal
-sim_FGenos <- function(founder_ids, RV_founder, RV_founder_pat,
+sim_FGenos <- function(founder_ids, RV_founder,
+                       founder_pat_allele, founder_mat_allele,
                        haplos, RV_col_loc, RV_pool_loc) {
 
-  #reduce haplos to contain only haplotypes that do
-  #not carry any of the cRVs in our pool of possible SNVs
-  no_CRVhaps <- condition_haplos_no_cSNV(haplos, RV_pool_loc)
+  #Determine which haplotypes carry the familial rare variant and which do not
+  #Determine which haplotypes carry the familial cRV
+  RV_hap_loc <- which(haplos[, RV_col_loc] == 1)
+
+  #Determine which haplotypes do not carry ANY crv in the pool
+  no_CRVrows <- find_no_cSNV_rows(haplos, RV_pool_loc)
+
+
 
   #here we handle the fully sporatic families
   #i.e. families that do not segregate any cSNVs
   #In this case, the haplotypes for ALL founders
   #is sampled from no_CRVhaps
   if(length(RV_founder) == 0){
-
     #sample all founder data from this pool
-    founder_genos <- no_CRVhaps[c(sample(x = 1:nrow(no_CRVhaps),
-                                       size = 2*length(founder_ids),
-                                       replace = TRUE)), ]
+    founder_genos <- haplos[sample(x = no_CRVrows,
+                                   size = 2*length(founder_ids),
+                                   replace = TRUE), ]
   } else {
 
-    #Determine which haplotypes carry the familial rare variant and which do not
-    RV_hap_loc <- which(haplos[, RV_col_loc] == 1)
+    #sample the paternally inherited founder haplotypes
+    pat_inherited_haps <- sapply(founder_pat_allele, function(x){
+      if(x == 0){
+        resample(x = no_CRVrows, size = 1)
+      } else {
+        resample(x = RV_hap_loc, size = 1)
+      }})
 
-    #NOTE: Under this scheme, marry-ins may NOT introduce any SNV
-    #from our pool of causal rare variants
-
-    #for the seed founder: sample one haplotype from those that carry the RV
-    #and one haplotype from those that DO NOT carry the RV
-    #for all other founders: sample 2 haplotypes that do not carry the RV
-    if(length(RV_hap_loc) == 1){
-      founder_genos <- rbind(haplos[RV_hap_loc, ],
-                             no_CRVhaps[c(sample(x = 1:nrow(no_CRVhaps),
-                                                 size = (2*length(founder_ids) + 1),
-                                                 replace = TRUE)), ])
-    } else {
-      founder_genos <- rbind(haplos[sample(x = RV_hap_loc, size = 1), ],
-                             no_CRVhaps[c(sample(x = 1:nrow(no_CRVhaps),
-                                                 size = (2*length(founder_ids) + 1),
-                                                 replace = TRUE)), ])
-    }
-
-    #Asscociate CRV to row 1, if paternally inherited OR row 2 if maternally inherited.
-    if(RV_founder_pat == 0){
-      founder_genos[c(1,2), ] <- founder_genos[c(2, 1), ]
-    }
+    #sample the maternally inherited founder haplotypes
+    mat_inherited_haps <- sapply(founder_mat_allele, function(x){
+      if(x == 0){
+        resample(x = no_CRVrows, size = 1)
+      } else {
+        resample(x = RV_hap_loc, size = 1)
+      }})
 
+    #pull the sampled haplotypes from the haplos matrix
+    founder_genos <- haplos[c(pat_inherited_haps, mat_inherited_haps), ]
   }
 
   #create IDs to associate founders to rows in founder_genos
-  founder_genos_ID <- rep(c(RV_founder, founder_ids), each = 2)
+  founder_genos_ID <- rep(founder_ids, 2)
+
+  #re-order so that founder haplotypes appear in order
+  founder_genos <- founder_genos[order(founder_genos_ID), ]
+  founder_genos_ID <- founder_genos_ID[order(founder_genos_ID)]
+
 
   return(list(founder_genos, founder_genos_ID))
 }
@@ -151,9 +190,11 @@ remove_allWild <- function(f_haps, SNV_map){
 #' \code{sex} \tab numeric \tab sex identification variable: \code{sex = 0} for males, and \code{sex = 1} females. \cr
 #' \code{dadID} \tab numeric \tab identification number of father \cr
 #' \code{momID} \tab numeric \tab identification number of mother \cr
-#' \code{affected} \tab logical \tab disease-affection status: \code{affected = TRUE} if individual has developed disease, and \code{FALSE} otherwise. \cr
-#' \code{DA1} \tab numeric \tab paternally inherited allele at the cRV locus: \code{DA1 = 1} if the cRV is inherited, and \code{0} otherwise. \cr
-#' \code{DA2} \tab numeric \tab maternally inherited allele at the cRV locus: \code{DA2 = 1} if the cRV is inherited, and \code{0} otherwise.\cr
+#' \code{affected} \tab logical \tab disease status indicator: set \code{affected = TRUE} if individual has disease.\cr
+#' \code{DA1} \tab numeric \tab paternally inherited allele at the cRV locus: \cr
+#'  \tab \tab  \code{DA1 = 1} if the cRV is inherited, and \code{0} otherwise. \cr
+#'  \code{DA2} \tab numeric \tab maternally inherited allele at the cRV locus: \cr
+#'  \tab \tab \code{DA2 = 1} if the cRV is inherited, and \code{0} otherwise.\cr
 #' }
 #'
 #' If \code{ped_files} does not contain the variables \code{DA1} and \code{DA2} the pedigrees are assumed to be fully sporadic.  Hence, the supplied pedigrees will not segregate any of the SNVs in the user-specified pool of cRVs.
@@ -165,24 +206,26 @@ remove_allWild <- function(f_haps, SNV_map){
 #' \strong{name} \tab \strong{type} \tab \strong{description} \cr
 #' \code{colID} \tab numeric \tab associates the rows in \code{SNV_map} to the columns of \code{haplos}\cr
 #' \code{chrom} \tab numeric \tab the chromosome that the SNV resides on\cr
-#' \code{position} \tab numeric \tab is the position of the SNV in base pairs when \code{pos_in_bp = TRUE} or centiMorgan when \code{pos_in_bp = FALSE}\cr
-#' \code{marker} \tab character \tab (Optional) a unique character identifier for the SNV. If missing this variable will be created from \code{chrom} and \code{position}. \cr
+#' \code{position} \tab numeric \tab is the position of the SNV in base pairs when argument \cr
+#' \tab \tab \code{pos_in_bp = TRUE} or centiMorgan when \code{pos_in_bp = FALSE}\cr
+#' \code{marker} \tab character \tab (Optional) a unique character identifier for the SNV. \cr
+#' \tab \tab If missing this variable will be created from \code{chrom} and \code{position}. \cr
 #' \code{pathwaySNV} \tab logical \tab (Optional) identifies SNVs located within the pathway of interest as \code{TRUE} \cr
-#' \code{is_CRV} \tab logical \tab  identifies causal rare variants (cRVs) as \code{TRUE}.  Note familial cRVs are sampled, with replacement from the SNVs for which \code{is_crv = TRUE}. \cr
+#' \code{is_CRV} \tab logical \tab  identifies causal rare variants (cRVs) as \code{TRUE}.\cr
 #' }
 #'
 #' Please note that when the variable \code{is_CRV} is missing from \code{SNV_map}, we sample a single SNV to be the causal rare variant for all pedigrees in the study, which is identified in the returned \code{famStudy} object.
 #'
 #' @param ped_files Data frame. A data frame of pedigrees for which to simulate sequence data, see details.
-#' @param haplos sparseMatrix. A sparse matrix of haplotype data, which contains the haplotypes for unrelated individuals representing the founder population.  Rows are assumed to be haplotypes, while columns represent SNVs.  If the \code{\link{read_slim}} function was used to import SLiM data to \code{R}, users may supply the sparse matrix \code{Haplotypes} returned by \code{read_slim}.
-#' @param SNV_map Data frame. A data frame that catalogs the SNVs in \code{haplos}.  If the \code{\link{read_slim}} function was used to import SLiM data to \code{R}, the data frame \code{Mutations} is of the proper format for \code{SNV_map}.  However, users must add the variable \code{is_CRV} to this data frame, see details.
+#' @param SNV_data SNVdata. An object of class \code{SNVdata} created by \code{\link{SNVdata}}.
 #' @param affected_only Logical. When \code{affected_only = TRUE}, we only simulate SNV data for the disease-affected individuals and the family members that connect them along a line of descent.  When \code{affected_only = FALSE}, SNV data is simulated for the entire study. By default, \code{affected_only = TRUE}.
 #' @param pos_in_bp Logical. This argument indicates if the positions in \code{SNV_map} are listed in base pairs.  By default, \code{pos_in_bp = TRUE}. If the positions in \code{SNV_map} are listed in centiMorgan please set \code{pos_in_bp = FALSE} instead.
-#' @param remove_wild Logical.  When \code{remove_wild = TRUE} the data is reduced by removing SNVs which are not observed in any of the study participants; otherwise if \code{remove_wild = FALSE} no data reduction occurs.  By default, \code{remove_wild = TRUE}.
-#' @param gamma_params Numeric list of length 2. The respective shape and rate parameters of the gamma distribution used to simulate distance between chiasmata.  By default, \code{gamma_params = c(2.63, 2*2.63)}, as discussed in Voorrips and Maliepaard (2012).
+#' @param remove_wild Logical.  When \code{remove_wild = TRUE} the data is reduced by removing SNVs which are not observed in any of the study participants; otherwise if \code{remove_wild} \code{= FALSE} no data reduction occurs.  By default, \code{remove_wild = TRUE}.
+#' @param gamma_params Numeric list of length 2. The respective shape and rate parameters of the gamma distribution used to simulate distance between chiasmata.  By default, \code{gamma} \code{_params} \code{= c(2.63, 2*2.63)}, as discussed in Voorrips and Maliepaard (2012).
 #' @param burn_in Numeric. The "burn-in" distance in centiMorgan, as defined by Voorrips and Maliepaard (2012), which is required before simulating the location of the first chiasmata with interference. By default, \code{burn_in = 1000}.
 #' The burn in distance in cM. By default, \code{burn_in = 1000}.
-#'
+#' @param SNV_map This argument has been deprecated. Users now supply objects of class \code{SNVdata} to argument \code{SNV_data}.
+#' @param haplos This argument has been deprecated. Users now supply objects of class \code{SNVdata} to argument \code{SNV_data}.
 #'
 #' @return  A object of class \code{famStudy}.  Objects of class \code{famStudy} are lists that include the following named items:
 #' @return \item{\code{ped_files}}{A data frame containing the sample of pedigrees for which sequence data was simulated.}
@@ -212,10 +255,13 @@ remove_allWild <- function(f_haps, SNV_map){
 #' EXmuts$is_CRV = FALSE
 #' EXmuts$is_CRV[c(26, 139, 223, 228, 472)] = TRUE
 #'
+#' # create object of class SNVdata
+#' my_SNVdata <- SNVdata(Haplotypes = EXhaps,
+#'                       Mutations = EXmuts)
+#'
 #' #supply required inputs to the sim_RVstudy function
 #' seqDat = sim_RVstudy(ped_files = study_peds,
-#'                      SNV_map = EXmuts,
-#'                      haplos = EXhaps)
+#'                      SNV_data = my_SNVdata)
 #'
 #'
 #' # Inbreeding examples
@@ -231,8 +277,7 @@ remove_allWild <- function(f_haps, SNV_map){
 #'
 #' # Notice that this instance of inbreeding can be accommodated by our model.
 #' seqDat = sim_RVstudy(ped_files = imb_ped1,
-#'                      SNV_map = EXmuts,
-#'                      haplos = EXhaps)
+#'                      SNV_data = my_SNVdata)
 #'
 #' # Create different type of inbreeding in family 1 of study_peds
 #' imb_ped2 <- study_peds[study_peds$FamID == 3, ]
@@ -243,22 +288,23 @@ remove_allWild <- function(f_haps, SNV_map){
 #' # error when the sim_RVstudy function is executed
 #' \dontrun{
 #' seqDat = sim_RVstudy(ped_files = imb_ped2,
-#'                      SNV_map = EXmuts,
-#'                      haplos = EXhaps)
+#'                      SNV_data = my_SNVdata)
 #' }
 #'
-sim_RVstudy <- function(ped_files, haplos, SNV_map,
+sim_RVstudy <- function(ped_files, SNV_data,
                         affected_only = TRUE,
                         remove_wild = TRUE,
                         pos_in_bp = TRUE,
                         gamma_params = c(2.63, 2.63/0.5),
-                        burn_in = 1000){
+                        burn_in = 1000,
+                        SNV_map = NULL, haplos = NULL){
 
-  #check SNV_map for possible issues
-  check_SNV_map(SNV_map)
+  if (!is.null(SNV_map) | !is.null(haplos)) {
+    stop("Arguments 'SNV_map' and 'haplos' have been deprecated.\n Instead, please supply to argument 'SNV_data' an object of class SNVdata.  Execute help(SNVdata) for more information." )
+  }
 
-  if (!"marker" %in% colnames(SNV_map)) {
-    SNV_map$marker <- make.unique(paste0(SNV_map$chrom, sep = "_", SNV_map$position))
+  if (!is.SNVdata(SNV_data)) {
+    stop("Expecting SNV_data to be an object of class SNVdata")
   }
 
   #check to see if DA1 and DA2 are both missing, if so
@@ -269,13 +315,21 @@ sim_RVstudy <- function(ped_files, haplos, SNV_map,
     warning("\n The variables DA1 and DA2 are missing from ped_files. \n Assuming fully sporadic ... \n...setting DA1 = DA2 = 0 for all pedigrees.")
   }
 
+
   #check ped_files for possible issues
   check_peds(ped_files)
 
-  if (nrow(SNV_map) != ncol(haplos)) {
-    stop("\n nrow(SNV_map) != ncol(haplos). \n SNV_map must catalog every SNV in haplos.")
+  #assign generation number if not included in ped_file
+  if(!"Gen" %in% colnames(ped_files)){
+    ped_files$Gen <- unlist(lapply(unique(ped_files$FamID), function(x){assign_gen(ped_files[ped_files$FamID == x, ])}))
   }
 
+  #quick and dirty assignment to see if new format creates problems down to line...
+  #TODO: change all function definitions to accept new SNVdata object
+  #NOTE: All of the same checks were performed on input data, the new SNVdata object
+  SNV_map = SNV_data$Mutations
+  haplos = SNV_data$Haplotypes
+
   #check to see that the sample contains affected relatives when the
   #affected_only setting is used
   if (affected_only & all(ped_files$affected  == FALSE)) {
@@ -296,7 +350,7 @@ sim_RVstudy <- function(ped_files, haplos, SNV_map,
     #the individuals who connect them along a line of descent.
     Afams <- lapply(FamIDs, function(x){
       affected_onlyPed(ped_file = ped_files[which(ped_files$FamID == x),])
-      })
+    })
 
     #combine the reduced pedigrees
     ped_files <- do.call("rbind", Afams)
@@ -347,14 +401,14 @@ sim_RVstudy <- function(ped_files, haplos, SNV_map,
   #haplotypes from conditional haplotype distribution
   f_genos <- lapply(c(1:length(FamIDs)), function(x){
     sim_FGenos(founder_ids = ped_files$ID[which(ped_files$FamID == FamIDs[x]
-                                                & is.na(ped_files$dadID)
-                                                & (ped_files$DA1 + ped_files$DA2) == 0)],
+                                                & is.na(ped_files$dadID))],
                RV_founder = ped_files$ID[which(ped_files$FamID == FamIDs[x]
                                                & is.na(ped_files$dadID)
-                                               & (ped_files$DA1 + ped_files$DA2) == 1)],
-               RV_founder_pat = ped_files$DA1[which(ped_files$FamID == FamIDs[x]
-                                                   & is.na(ped_files$dadID)
-                                                   & (ped_files$DA1 + ped_files$DA2) == 1)],
+                                               & (ped_files$DA1 + ped_files$DA2) != 0)],
+               founder_pat_allele = ped_files$DA1[which(ped_files$FamID == FamIDs[x]
+                                                        & is.na(ped_files$dadID))],
+               founder_mat_allele = ped_files$DA2[which(ped_files$FamID == FamIDs[x]
+                                                        & is.na(ped_files$dadID))],
                haplos, RV_col_loc = which(SNV_map$marker == Fam_RVs[x]),
                RV_pool_loc = SNV_map$colID[SNV_map$is_CRV])
   })
@@ -383,11 +437,11 @@ sim_RVstudy <- function(ped_files, haplos, SNV_map,
   #simulate non-founder haploypes via conditional gene drop
   ped_seqs <- lapply(c(1:length(FamIDs)), function(x){
     sim_seq(ped_file = ped_files[ped_files$FamID == FamIDs[x], ],
-              founder_genos = f_genos[[x]],
-              SNV_map, chrom_map,
-              RV_marker = Fam_RVs[x],
-              burn_in, gamma_params)
-    })
+            founder_genos = f_genos[[x]],
+            SNV_map, chrom_map,
+            RV_marker = Fam_RVs[x],
+            burn_in, gamma_params)
+  })
 
   ped_haplos <- do.call("rbind", lapply(ped_seqs, function(x){x$ped_genos}))
   haplo_map <- do.call("rbind", lapply(ped_seqs, function(x){x$geno_map}))
